import pygame

#from pykitten import utils
from pykitten.gui import ClickArea

from stuff import *
from menus import PopUpMenu
from widgets import *
from unitdat import *
from troops import *


def sim_fight():
    def gen_troop(troop, color):
        tr = {}
        for type in troop.keys():
            tr[type] = []
            for i in xrange(troop[type]):
                tr[type].append(UNIT_TYPES[type](color))
        return tr

    troop1 = {
        "Soldier": 1,
        "Flamethrower": 0,
        "Grenadier":0,
        "Tank": 0,
        "Jet": 0
    }

    troop2 = {
        "Soldier": 1,
        "Flamethrower": 0,
        "Grenadier": 0,
        "Tank": 0,
        "Jet": 0,
    }

    troop1 = gen_troop(troop1, "red")
    troop2 = gen_troop(troop2, "blue")
    calc_fight(troop1, troop2, "red", "blue")

def calc_fight(troop1, troop2, col1, col2):
    def make_move(t, enemies):
        for unit in t:
            targets = unit.get_target(enemies)
            if targets:
                unit.attack(targets)

    def clean_up(troop):
        #remove dead units
        tmp = troop.copy()
        for type in tmp:
            for unit in tmp[type]:
                if unit.hp <= 0:
                    killcount[unit.color][type] += 1
                    troop[type].remove(unit)


    def check_alive(troop):
        for type in troop.keys():
            for unit in troop[type]:
                if unit.hp >= 0:
                    return True
        return False

    killcount = {}
    killcount[col1] = {"Soldier": 0, "Flamethrower": 0, "Grenadier": 0,
        "Tank": 0, "Jet": 0, }
    killcount[col2] = {"Soldier": 0, "Flamethrower": 0, "Grenadier": 0,
        "Tank": 0, "Jet": 0, }

    troop1 = troop1.copy()
    troop2 = troop2.copy()
    for type in INITIATIVE:
        t1, t2 = [], []
        if type in troop1.keys():
            t1.extend(troop1[type])
        if type in troop2.keys():
            t2.extend(troop2[type])

        make_move(t1, troop2)
        make_move(t2, troop1)

        clean_up(troop1)  #remove dead units
        clean_up(troop2)

    tr1_alive = check_alive(troop1)
    tr2_alive = check_alive(troop2)
    #print
    #print "Troop 1 alive:", tr1_alive, "- Losses:", killcount[col1]
    #print "Troop 2 alive:", tr2_alive, "- Losses:", killcount[col2]

    return tr1_alive, tr2_alive, killcount


class Fight(PopUpMenu):
    def __init__(self, game):
        def show_troop(t, xcenter, y):
            y_add = 41
            for type in TYPES:
                nr = 0
                if t.has_key(type):
                    nr = len(t[type])
                img = FONT["small"].print_line(nr)
                x = xcenter - (img.get_size()[0]/2)
                self.bg.blit(img, (x,y))
                y += y_add

        def show_kills(t, xcenter, y):
            y_add = 41
            for type in TYPES:
                nr = 0
                if t.has_key(type):
                    nr = t[type]
                img = FONT["small"].print_line(nr)
                x = xcenter - (img.get_size()[0]/2)
                self.bg.blit(img, (x,y))
                y += y_add

        PopUpMenu.__init__(self, game.screen, GFX["battle.png"])

        self.hq_conquered = False
        self.game = game
        reg1 = game.from_reg
        reg2 = game.dest_reg
        troop1 = game.cur_troops
        troop2 = reg2.troops

        sx, sy = GFX["battle.png"].get_size()
        wx, wy = utils.center((sx,sy), (800,600))

        text = FONT["small"].print_line(reg1.name)  #Region names
        x = 76 + wx - (text.get_size()[0]/2)  #centered on pixel (76,22)
        y = 22+wy #+ 5
        self.bg.blit(text, (x,y))
        text = FONT["small"].print_line(reg2.name)
        x = 265 + wx - (text.get_size()[0]/2)
        self.bg.blit(text, (x,y))

        flag = FLAG[reg1.owner + "_m"]  #flags
        self.bg.blit(flag, (10+wx,30+wy))
        flag = FLAG[reg2.owner + "_m"]
        self.bg.blit(flag, (199+wx,30+wy))

        ty = 192+wy
        show_troop(troop1, 41+wx, ty)
        show_troop(troop2, 230+wx, ty)

        alive1, alive2, deaths = calc_fight(troop1, troop2, reg1.owner,
            reg2.owner)


        #remove dead units from regions
        for type in reg1.troops:
            for unit in reg1.troops[type]:
                if unit.hp <= 0:
                    reg1.troops[type].remove(unit)

        for type in reg2.troops:
            for unit in reg2.troops[type]:
                if unit.hp <= 0:
                    reg2.troops[type].remove(unit)

        #battle aftermath
        show_kills(deaths[reg1.owner], 111+wx, ty)
        show_kills(deaths[reg2.owner], 300+wx, ty)

        if alive1 and not alive2:  #attacker wins and takes over region
            msg = "victory.png"
            self.win = 1
            reg2.rem_troops()
            reg1.rem_troops(troop1)
            reg2.add_troops(troop1)
            game.map.select(reg2.id)

            for name in game.players.keys():  #remove region from other player
                if reg2.id in game.players[name].regnames:
                    game.players[name].rem_region(reg2)

            game.player.add_region(reg2)  #add region to attacker
            game.player.calc_gain()
            if reg2.hq:
                self.hq_conquered = 1
        elif not alive1 and alive2:  #attacker lost all units
            msg = "defeat.png"
            self.win = -1
        else:
            msg = "draw.png"  #both sides are dead or still alive(TM)
            self.win = 0

        x = utils.center(GFX[msg].get_size(), (800,600))[0]
        self.bg.blit(GFX[msg], (x,389+wy))

        game.cur_troops = None
        game.from_reg = None
        game.dest_reg = None

        button = NamedFlipButton("confirm", callback=self.quit)
        button.set_pos((wx+145, wy+409))
        self.widgets.append(button)

    def main_start(self):
        if self.win > 0:
            SFX["bwin"].play()
        elif self.win < 0:
            SFX["blose"].play()
        else:
            SFX["bdraw"].play()
        PopUpMenu.main_start(self)


    def update_screen(self):
        changed = 0
        for w in self.widgets:
            if w.changed:
                changed = 1
                w.changed = 0
                w.draw(self.bg)

        if changed:
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()











if __name__=='__main__':
    sim_fight()
