import random

from player import *
from unitdat import *
from troops import *
from fight import calc_fight

# very simplistic game ai
class LittleShodan(Player):
    def __init__(self, color, game):
        Player.__init__(self, color, ai=True)
        self.game = game
        self.movements = []

    def make_move(self):
        self.move_troops()  #first move (new towns may be conquered..)
        self.build()  #then build

    def build(self):
        build_prob = 0.5

        for reg in self.regions:
            if (reg.town or reg.hq) and build_prob > random.random():
                pu = reg.pu
                new_troops = []
                min=pu/100

                for i in xrange(5):
                    av = self.calc_available(pu)
                    type = TYPES[i]
                    if av[i]>0:
                        cost=min
                        if av[i] >= min:
                            new_troops.append(min)
                            #print "Normal:", reg.name, "builds", min, type
                        else:
                            new_troops.append(av[i])
                            cost=av[i]
                            #print "Normal:", reg.name, "builds", av[i], type
                        pu -= unit_dat[type]["p.u."] * cost
                        for j in xrange(3):  #rem res
                            res = unit_dat[type]["cost"][j] * cost
                            self.res[j] -= res
                    else:
                        new_troops.append(0)

                #new_troops.reverse()
                for i in xrange(4, -1, -1):  #build most expensive units first
                    av = self.calc_available(pu)
                    type = TYPES[i]

                    if av[i]>0:
                        new_troops[i] += av[i]
                        #print "Filling:", reg.name, "builds", av[i], type
                        pu -= unit_dat[type]["p.u."] * av[i]
                        for j in xrange(3):  #rem res
                            res = unit_dat[type]["cost"][j] * av[i]
                            self.res[j] -= res

                #new_troops.reverse()
                self.cur_res = self.res[:]
                reg.new_troops = new_troops
                #print "Total:", reg.name, "builds", new_troops

    def move_troops(self):
        movements = []
        for reg in self.regions:
            if reg.occupied():
                dest = self.find_dest(reg.id)
                if dest:
                    movements.append((reg, dest, reg.troops))
        self.movements = movements


    def find_dest(self, reg):
        dest = None
        region = self.game.map.region
        for id in region[reg].adjacent:
            if region[id].owner != self.color:
                dest = id
                break

        if not dest:
            dest  = random.randrange(0, len(region[reg].adjacent))
            dest = region[reg].adjacent[dest]
        return region[dest]

    def calc_available(self, pu):
        avail = []
        for type in TYPES:
            #max no. of units according to p.u.
            unit_pu = unit_dat[type]["p.u."]
            max = pu / unit_pu
            for i in xrange(3):  #max according to resources
                if max==0:
                    break
                c = unit_dat[type]["cost"][i]
                if c > 0:
                    new_max = self.res[i] / c
                    if new_max < max:
                        max = new_max
            avail.append(max)
        return avail


    def ai_move_troops(self):
        def takeover():
            for name in self.game.players.keys():  #remove region from other player
                if dest.id in self.game.players[name].regnames:
                    self.game.players[name].rem_region(dest)

            dest.rem_troops()
            dest.add_troops(troops)
            from_reg.rem_troops(troops)
            self.add_region(dest)
            self.calc_gain()

        def rem_dead(reg):
            for type in reg.troops:
                for unit in reg.troops[type]:
                    if unit.hp <= 0:
                        reg.troops[type].remove(unit)

        from_reg, dest, troops = self.movements.pop()

        tr = troops.copy()
        for type in troops:
            for unit in troops[type]:
                if unit.moved:
                    tr[type].remove(unit)
        for type in troops:
            for unit in troops[type]:
                unit.moved = True
        troops = tr

        #move to dest
        if self.color==dest.owner:  #merge with own troops
            #dest.rem_troops()
#            print "\nMerging:",
#            for type in TYPES:
#                print len(dest.troops[type]),
#            print "\nWith:",
#            for type in TYPES:
#                print len(troops[type]),

            dest.add_troops(troops)

#            print "\nTotal:",
#            for type in TYPES:
#                print len(dest.troops[type]),

            from_reg.rem_troops(troops)

        elif not dest.occupied():
            color = dest.owner
            takeover()
            if color != "grey":
                report = {}
                report["battle"] = False #no fighting took place
                report["from"] = from_reg.name
                report["dest"] = dest.name
                report["attacker"] = self.color
                report["defender"] = color
                return report
        else:
            #fight
            tr1_alive, tr2_alive, killcount = calc_fight(troops, dest.troops,
                self.color, dest.owner)
            rem_dead(from_reg)
            rem_dead(dest)

            color = dest.owner
            if tr1_alive and not tr2_alive:
                win = 1
                takeover()
            elif not tr1_alive and tr2_alive:
                win = -1 #loss
            else:
                win = 0  #draw

            if color != "grey":
                report = {}
                report["battle"] = True
                report["destid"] = dest.id
                report["from"] = from_reg.name
                report["dest"] = dest.name
                report["attacker"] = self.color
                report["defender"] = color

                killed = 0 #kills by defender
                lost = 0  #losses of defender
                for type in TYPES:
                    lost += killcount[color][type]
                    killed += killcount[self.color][type]

                report["killed"] = str(killed)
                report["lost"] = str(lost)
                if win > 0:  #shodan's win is our loss!
                    report["win"] = -1
                elif win==0:
                    report["win"] = 0
                else:
                    report["win"] = 1

                return report

        return None

