import pygame, random
from pygame.locals import *

from pykitten import utils
from pykitten.gui import ClickArea

from stuff import *
from menus import *
from widgets import *
from player import Player
from states import Game

class Logo(FadeState):
    def __init__(self):
        FadeState.__init__(self, 0)
        img = GFX["logo.png"].copy()
        pos = utils.center(img.get_size(), (800,600))
        self.bg.blit(img, pos)
        self.fading = 0.025
        #self.fps = 30
        JB.play("logo.ogg", rep=0)
        
        clicky = ClickArea(pygame.Rect([0,0,800,600]), 
            callback = self.to_menu)
        self.widgets.append(clicky)
        
        self.add_kbevent(KEYDOWN, K_RETURN, callback=self.to_menu)
        self.add_kbevent(KEYDOWN, K_ESCAPE, callback=self.to_menu)
        

    def update_other(self):
        if self.fading > 0 and self.alpha>7.6:
            self.fading = 2
        
    def after_fadein(self):
        self.fading = -2.4
    
    def after_fadeout(self):
        self.to_menu()
    
    def to_menu(self):
        JB.stop()
        self.next = StartMenu()
        self.quit()
            

class StartMenu(FadeState):
    def __init__(self, screen=None):
        FadeState.__init__(self)
        if screen:
            self.screen.blit(screen, (0,0))
        self.bg.blit(GFX["main.png"], (0,0))
        self.fading = 2
        self.fps = 30
        self.choice = None
                
        #340,404/441/512
        #start game button
        button = NamedFlipButton("main_start", callback=self.start_game)
        self.add_button(button, (339,403), "start")
       
        #restore button
        button = NamedFlipButton("main_restore", callback=self.choose, 
            args=1)
        self.add_button(button, (339,440), "restore")

        
        #quit game button
        button = NamedFlipButton("main_quit", callback=self.choose, 
            args=2)
        self.add_button(button, (339,511), "quit")

        #57x38 50/697,410
        clicky = ClickArea(pygame.Rect([50,410,57,38]),
            callback=self.choose, arg=3)
        self.widgets.append(clicky)
        
        clicky = ClickArea(pygame.Rect([697,410,57,38]),
            callback=self.choose, arg=3)
        self.widgets.append(clicky)    
        
        self.alpha = 0
        self.bg.set_alpha(0)
        
    def choose(self, nr):
        self.choice =  nr   
        self.fading = -15

    def after_fadeout(self):
        if self.choice==1:
            self.restore()
        elif self.choice==2:
            self.quit_game()
        elif self.choice==3:
            self.credits()
    
    def quit_game(self):
        self.kill()
        
    def credits(self):
        self.next = Credits()
        self.quit()
    
    def start_game(self):
        self.next = ChoosePlayers(self.screen)
        self.quit()
    
    def restore(self):
        self.next = Game(savegame = True)
        self.pause()
        self.quit()
        
    def unpause(self):
        FadeState.unpause(self)
        self.next = StartMenu()
        self.quit()

    
    def quit_game(self):
        print "Thanks for playing! :)"
        self.quit()
        

class ChoosePlayers(FadeState):
    def __init__(self, screen):
        FadeState.__init__(self)
        self.bg.blit(GFX["main.png"], (0,0))
        self.screen.blit(screen, (0,0))
        self.fading = 2
        self.alpha = 0
        
        #340,404/441/512
        #1 player
        button = NamedFlipButton("main_onepl", callback=self.start_one)
        self.add_button(button, (339,403), "one")
        #self.widgets.append(button)

        #2 player
        button = NamedFlipButton("main_twopl", callback=self.start_two)
        self.add_button(button, (339,440), "two")
        #self.widgets.append(button)
        
        #back to main
        button = NamedFlipButton("main_back", callback=self.to_main)
        self.add_button(button, (339,511), "back")
        #self.widgets.append(button)
        
        #57x38 50/697,410
        clicky = ClickArea(pygame.Rect([50,410,57,38]),
            callback=self.choose, arg=3)
        self.widgets.append(clicky)
        
        clicky = ClickArea(pygame.Rect([697,410,57,38]),
            callback=self.choose, arg=3)
        self.widgets.append(clicky)  
        
    def start_one(self):
        self.fading = -15
        self.choice = 1
        
    def start_two(self):
        self.fading = -15
        self.choice = 2
    
    def choose(self, nr):
        self.choice = nr
        self.fading = -15
        
    def after_fadeout(self):
        if self.choice==3:
            self.next = Credits()
        else:
            self.next = ChooseFeatheration(self.choice)
        self.quit()

    def to_main(self):
        self.next = StartMenu()
        self.quit()

class ChooseFeatheration(FadeState):
    def __init__(self, nr):
        FadeState.__init__(self)
        self.playernr = nr
        self.bg = GFX["choose.png"].copy()
        self.fading = 10
        
        #300x195 68,222
        clicky = ClickArea(pygame.Rect([68,222,300,195]), 
            callback=self.blue)
        self.widgets.append(clicky)
        clicky = ClickArea(pygame.Rect([430,222,300,195]),
            callback=self.red)
        self.widgets.append(clicky)
        
    def red(self):
        self.fading = -15
        self.choice = 1
    
    def blue(self):
        self.choice = 0
        self.fading = -15
        
    def after_fadeout(self):
        color1 = "blue"  #player 1
        color2 = "red"  #player 2
        if self.choice: 
            color1 = "red"
            color2 = "blue" 
            
        players = {}
        players[color1] = Player(color1)
        if self.playernr == 2:
            players[color2] = Player(color2)
        
        self.pause()
        self.next = Game(players=players)
        self.quit()            

    def unpause(self):
        FadeState.unpause(self)
        self.next = StartMenu()
        self.quit()
        
class Credits(FadeState):
    def __init__(self):
        FadeState.__init__(self)
        self.bg = GFX["credits.png"].copy()
        self.fading = 1
        clicky = ClickArea(pygame.Rect([0,0,800,600]), 
            callback=self.to_menu)
        self.widgets.append(clicky)
        
    def to_menu(self):
        self.fading = -15
    
    def after_fadeout(self):
        self.next = StartMenu()
        self.quit()



