#import maps
import map01


from pykitten import utils
from pykitten.gui import *
from pykitten.data import *
from pykitten.engine import *

from report import *
from shodan import LittleShodan
from fight import Fight
from menus import *
from unitdat import *
from winlose import WinLose
from troops import *
from map import *
from player import Player
from widgets import *
from stuff import *
from saveload import *

class Game(BaseState):
    def __init__(self, players={"red":Player("red")}, map=map01, \
        savegame=None, p1="red"):
        if not pygame.mixer.music.get_busy():
            JB.play("bgm.ogg", -1)
        BaseState.__init__(self)
        self.conti = GameContainer(self)
        self.next_was = None
        self.bg.blit(GFX["interface.png"], (0,0))

        self.cur_troops = None #troops to be moved..
        self.from_reg = None #from this region..
        self.dest_reg = None #into that one..
        self.reports = []


        if savegame:
            game_data = load_game()
            if not game_data:
                savegame = False
                self.to_start_menu()

        if savegame:
            self.load_game()
            self.map.changed = 1
            for id in self.map.region:
                self.map.region[id].gen_image()
            self.map.select(self.map.hq[self.player.color])
        else:
            #load map
            self.factions = map.factions
            if p1 in self.factions:
                self.factions.remove(p1)
            self.factions.insert(0, p1)
            self.map = Map(self, map)
            self.bg.blit(self.map, self.map.pos)

            #setup players
            self.players = players
            for f in self.factions:
                start = self.map.region[self.map.hq[f]]
                if f not in self.players.keys():  #add ai player
                    self.players[f] = LittleShodan(f, self)
                self.players[f].add_region(start)
            self.turn = 0
            self.player = None  #current player

        self.bg.blit(self.map, self.map.pos)
        self.widgets.extend(self.map.region.values())

        #message line
        self.msgline = MsgLine(msg="", size=(270,5))
        self.msgline.set_pos((165,590))
        self.widgets.append(self.msgline)

        #fizzlefizzle
        self.fizzle = Fizzle(images =[GFX["comm_fizzle1.png"],
            GFX["comm_fizzle2.png"]])
        self.fizzle.set_pos((657,3))

        #move unit button
        button = NamedFlipButton("move", callback=self.select_dest, img4=True)
        self.add_button(button, (730,560), "move")
        self.widgets.append(button)

        #end turn button
        button = NamedFlipButton("endturn", callback=self.end_turn)
        self.add_button(button, (452,565), "turn")
        self.widgets.append(button)

        #build button
        button=NamedFlipButton("build",callback=self.to_build_menu,img4=True)
        self.add_button(button, (10,561), "build")
        self.widgets.append(button)


        #options button  - removed
        #button = NamedFlipButton("options", callback=self.none)
        #self.add_button(button, (x,y), "options")
        #self.widgets.append(button)

        #save button
        x, y = 4, 3
        button = NamedFlipButton("save", callback=save_game, args=self)
        self.add_button(button, (x,y), "save")
        self.widgets.append(button)

        #load button
        x += 65
        button = NamedFlipButton("load", callback=self.reload_game)
        self.add_button(button, (x,y), "load")
        self.widgets.append(button)

        #exit button
        x += 65
        button = NamedFlipButton("exit", callback=self.to_start_menu)
        self.add_button(button, (x,y), "exit")
        self.widgets.append(button)

    def start_turn(self):
        if self.player:
            for reg in self.player.regions:
                for type in TYPES:
                    for tr in reg.troops[type]:
                        tr.moved = 0

        self.player = self.players[self.factions[self.turn]]
        hq = self.map.hq[self.player.color]  #hq still there?
        if self.map.region[hq].owner != self.player.color:
            self.to_winlose(win=False)
            return

        for reg in self.player.regions:
            if reg.new_troops:
                reg.add_new_troops()

        self.player.calc_gain()
        self.player.calc_res()

        if self.player.ai:
            self.player.make_move()

        self.map.redraw()
        self.map.select(hq)
        self.conti.update()

        #show reports - if there are any.
        if not self.player.ai and self.reports:
            self.to_report()


    def select_dest(self):
        if not self.from_reg and not self.player.ai:
            self.button["move"].flip(0)

            self.from_reg = self.map.selected
            self.msgline.set_msg("Select destination.")
            self.map.changed = 1

    def set_dest(self, dest):
        if self.player.ai:
            return

        self.msgline.set_msg("")
        if dest.id in self.from_reg.adjacent:  #adjacent region selected
            self.dest_reg = dest
            self.to_move_menu()
            return True
        #print "Please select a region next to the troops"
        self.from_reg = None
        return False

    def move_troops(self):
        def takeover():
            for name in self.players.keys():  #remove region from other player
                if dest.id in self.players[name].regnames:
                    self.players[name].rem_region(dest)

            dest.rem_troops()
            dest.add_troops(self.cur_troops)
            self.from_reg.rem_troops(self.cur_troops)
            self.player.add_region(dest)
            self.player.calc_gain()
            for id in dest.adjacent:
                self.map.region[id].gen_image()

            dest.new_troops = [] #delete new troops
        dest = self.dest_reg

        #move to dest
        fighting = 0
        if self.player.color==dest.owner:  #merge with own troops
            dest.add_troops(self.cur_troops)
            self.from_reg.rem_troops(self.cur_troops)
        else:
            fighting = 1
            self.fight(dest)  #fight!

        self.conti.update()

        if not fighting:
            self.cur_troops = None
            self.from_reg = None

    def fight(self, dest):
        #print "Fight! .. You win!!!"
        #return True
        self.pause()
        self.next = Fight(self)
        self.next_was = 1
        self.quit()

    def end_turn(self):
        self.turn += 1
        if self.turn == len(self.factions):
            self.turn = 0
        #TODO: play sound, show switch animation.. whatever
        #pygame.time.wait(500)
        self.start_turn()

    def to_move_menu(self):
        self.pause()
        self.next = Move(self)
        self.next_was = 0
        self.quit()

    def to_start_menu(self):
        pygame.mixer.music.fadeout(20)
        self.quit()

    def to_build_menu(self):
        self.pause()
        self.next = Build(self)
        self.quit()

    def to_options_menu(self):
        self.pause()
        self.next = Options(self)
        self.quit()

    def to_winlose(self, win):
        self.pause()
        pygame.mixer.music.fadeout(20)
        self.next_was = 3
        self.next = WinLose(self.screen, win)
        self.quit()

    def to_report(self):
        self.pause()
        self.next = Report(self)
        self.quit()

    def reload_game(self):
        self.next = Game(savegame = True)
        self.quit()

    def load_game(self):
        game_data = load_game()
        if not game_data:
            self.quit()
            return

        self.map = Map(self, None, game_data)
        self.turn = game_data["turn"]
        self.factions = game_data["factions"]
        self.players = {}
        for id in game_data["players"]:
            p = game_data["players"][id]
            if not p.ai:
                player = Player(p.color)
            else:
                player = LittleShodan(p.color, self)
            player.regnames = p.regnames
            for name in p.regnames:
                player.regions.append(self.map.region[name])
            player.res = p.res
            player.cur_res = p.cur_res
            player.res_gain = p.res_gain
            if player.ai:
                player.movements = p.movements
            self.players[id] = player
        self.player = self.players[self.factions[self.turn]]

    def unpause(self):
        if self.next_was==0:  #previous state was Move-menu
            self.next_was = None
            self.cur_troops = self.next.troops
        elif self.next_was==1: #prev state was Fight
            self.next_was = None
            if self.next.hq_conquered:
                self.to_winlose(win=True)
                return
            self.map.redraw()
        elif self.next_was==3:  #win/lose screen
            State.unpause(self)
            self.to_start_menu()
            return

        State.unpause(self)

        if self.player.ai:
            return

        self.map.changed = 1
        self.conti.update()
        if self.cur_troops:
            self.move_troops()
        else:
            if self.from_reg:
                self.map.select(self.from_reg.id)
            self.from_reg = None
            self.dest_reg = None
        self.msgline.set_msg("")
        self.msgline.draw(self.bg)

    def update_other(self):
        if pygame.mouse.get_pressed()[2] and self.from_reg:
            self.from_reg = None
            self.cur_troops = None
            self.msgline.set_msg("")
            self.msgline.draw(self.bg)

        if self.player.ai and self.player.movements:
            rep = self.player.ai_move_troops()
            if rep:
                self.reports.append(rep)
        elif self.player.ai and not self.player.movements:
            self.dest_reg = None
            self.from_reg = None
            self.cur_troops = None
            self.map.redraw()
            self.map.changed = 1
            self.conti.update()

            self.end_turn()

    def update_screen(self):
        changed = 0
        if self.player.ai:
            return

        for name in self.button.keys():
            changed = 1
            self.button[name].draw(self.bg)

        if self.map.changed:
            changed = 1
            self.map.changed = 0
            if self.map.clicked:
                self.map.clicked = False
                if not self.from_reg:
                    self.show_reginfo()
            self.msgline.draw(self.bg)
            self.bg.blit(self.map, self.map.pos)

        if changed:
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()

    def show_reginfo(self):
        self.conti.update()

    def adjacent(self, reg, reglist):
        # check if reg is next to any region in reglist
        for r in reglist:
            if r in reg.adjacent:
                return True
        return False

    def main_start(self):
        if not self.player:
            self.start_turn()

    def none(self):
        pass


class Move(PopUpMenu):
    def __init__(self, game):
        PopUpMenu.__init__(self, game.screen, GFX["movement.png"])
        reg = game.from_reg
        self.msgline = game.msgline
        self.msgline.set_msg("Select troops")
        self.msgline.draw(self.bg)


        self.avail_troops = reg.get_available()
        self.region = reg
        self.sliders = []
        self.troops = {}

        wsiz = GFX["movement.png"].get_size()
        wpos = utils.center(wsiz,(800,600))


        flag = FLAG[game.from_reg.owner + "_t"]
        self.bg.blit(flag, (wpos[0]+28,wpos[1]+8))
        line = FONT["small"].print_line(game.from_reg.name)
        self.bg.blit(line, (wpos[0]+15,wpos[1]+22))

        flag = FLAG[game.dest_reg.owner + "_t"]
        self.bg.blit(flag, (wpos[0]+wsiz[0]-28-flag.get_size()[0],wpos[1]+8))
        line = FONT["small"].print_line(game.dest_reg.name)
        self.bg.blit(line, (wpos[0]+wsiz[0]-15-line.get_size()[0],wpos[1]+22))

        x = wpos[0] + 49
        y = wpos[1] + 46 +5
        for type in TYPES:
            max = len(self.avail_troops[type])
            s = MoveSlider(max, 0)
            s.set_pos((x,y))

            l1 = SliderLinkedInfo((20,5))  #number of units left on left side
            l1.set_pos((x-25, y+8))
            s.add_link(l1)
            l2 = SliderLinkedInfo((20,5), "right") #units on right side
            l2.set_pos((s.get_pos()[0]+s.get_size()[0]+5, y+8))
            s.add_link(l2)

            self.sliders.append(s)
            self.widgets.append(s)
            y += 51

        #submit button
        button = NamedFlipButton("confirm", callback=self.submit)
        self.add_button(button, (wpos[0]+34,wpos[1]+284), "submit")

        #cancel button
        button = NamedFlipButton("cancel", callback=self.abort)
        self.add_button(button, (wpos[0]+151,wpos[1]+284), "cancel")

        #all button
        x, y = 111+wpos[0],277+wpos[1]
        img2 = GFX["all.png"]
        tmp = img2.copy()
        tmp.set_colorkey((255,255,0))
        img1 = pygame.Surface(img2.get_size())
        img1.fill((0,0,0))
        img1.blit(tmp, (0,0))
        button = FlipButton(img1, img2, self.select_all)
        self.all = False
        self.add_button(button, (x, y), "all")

    def select_all(self):
        SFX["sldclick"].play()
        if self.all:
            self.all = False
            for s in self.sliders:
                s.set_cur(0)
        else:
            self.all = True
            for s in self.sliders:
                s.set_cur(s.max)


    def submit(self):
        troops = {}
        troops["Soldier"] = self.sliders[0].cur
        troops["Flamethrower"] = self.sliders[1].cur
        troops["Grenadier"] = self.sliders[2].cur
        troops["Tank"] = self.sliders[3].cur
        troops["Jet"] = self.sliders[4].cur

        units = self.avail_troops
        for t in TYPES:
            nr = troops[t]
            if nr > 0:
                move_troop =  units[t][:nr]
                self.troops[t] = move_troop
                for mt in move_troop:
                    mt.moved = True
        self.msgline.set_msg("")
        self.quit()

    def update_screen(self):
        change = 0
        for name in self.button.keys():
            change = 1
            self.button[name].draw(self.bg)

        for s in self.sliders:
            if s.changed:
                change = 1
                s.changed = 0
                s.draw(self.bg)
                for link in s.linked:
                    link.draw(self.bg)

        if change:
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()


class Build(PopUpMenu):
    def __init__(self, game):
        PopUpMenu.__init__(self, game.screen, GFX["build.png"])
        self.game = game

        reg = self.game.map.selected
        self.region = reg
        self.pu = reg.pu
        self.cur_pu = reg.pu
        self.cur_res = [0,0,0]
        self.res_gain = game.player.res_gain
        self.avail = [0,0,0,0,0]
        self.res_cost = [0,0,0]

        self.max_res = self.game.player.cur_res[:]
        self.troops = []  #new_troops to be build here
        troops = self.region.new_troops #new_troops already ordered in region
        if troops:
            self.troops.extend(troops)
            for i in xrange(len(TYPES)):
                for res in xrange(3):  #restore old max_res ..... stupid code!
                    cost = unit_dat[TYPES[i]]["cost"][res] * troops[i]
                    self.max_res[res] += cost
        else:
            for type in TYPES:
                self.troops.append(0)

        self.update_res()
        self.update_available()

        if troops:
            for i in xrange(len(TYPES)):
                self.avail[i] += troops[i]

        #draw ui elements
        self.conti = BuildContainer(self)
        self.msgline = game.msgline
        self.msgline.set_msg("")
        self.msgline.draw(self.bg)
        self.offset = utils.center(GFX["build.png"].get_size(), (800,600))
        wx, wy = self.offset

        #submit button
        button = NamedFlipButton("confirm", callback=self.submit)
        self.add_button(button, (206+wx,227+wy), "submit")
        self.widgets.append(button)

        #cancel button
        button = NamedFlipButton("cancel", callback=self.abort)
        self.add_button(button, (268+wx,227+wy), "abort")
        self.widgets.append(button)

        #troop cost info
        pos = [(85,36), (85,113), (85,190), (259,36), (259,113)]
        for i in xrange(len(TYPES)):
            x, y = pos[i]
            for price in unit_dat[TYPES[i]]["cost"]:
                img = FONT["small"].print_line(price)
                self.bg.blit(img, (x+wx, y+wy))
                y += 16
            img = FONT["small"].print_line(unit_dat[TYPES[i]]["p.u."])
            self.bg.blit(img, (x+wx, y+wy))

        #plus and minus buttons
        pos = [(110,42), (110,119), (110,196), (284,42), (284,119)]
        snd = SFX["sldclick"]
        btns = ["plusplus", "plus", "minus", "minusminus"]
        for i in xrange(len(TYPES)):
            x, y = pos[i]
            for name in btns:
                img2 = GFX["btn_"+name+".png"]
                tmp = img2.copy()
                tmp.set_colorkey((255,255,0))
                img1 = pygame.Surface(img2.get_size())
                img1.fill((0,0,0))
                img1.blit(tmp, (0,0))
                id = (i, name)
                button = FlipButton(img1, img2, callback=self.clicked, \
                    args=id, snd=snd)
                self.add_button(button, (x+wx, y+wy), id)
                y += 11
        self.conti.update()

    def clicked(self, id):
        nr, name = id
        if name=="plus" and self.troops[nr] < self.avail[nr]:
            self.troops[nr] += 1
        elif name=="minus" and self.troops[nr] > 0:
            self.troops[nr] -= 1

        if name=="plusplus":
            self.troops[nr] = self.avail[nr]
        elif name=="minusminus":
            self.troops[nr] = 0

        self.update_res()
        self.update_available(nr, True)

    def update_res(self):
        self.res_cost = [0,0,0]
        pu = self.pu
        for i in xrange(len(TYPES)):
            type = TYPES[i]
            pu_cost = unit_dat[type]["p.u."]
            pu -= self.troops[i]*pu_cost

            cost = unit_dat[type]["cost"]
            for j in xrange(3):
                self.res_cost[j] += self.troops[i]*cost[j]

        self.cur_pu = pu
        for i in xrange(3):
            self.cur_res[i] = self.max_res[i] - self.res_cost[i]

    def update_available(self, nr=None, decreased=False):
        avail = self.avail
        for i in xrange(len(TYPES)):
            if nr != i:
                max = []
                type = TYPES[i]
                unit_pu = unit_dat[type]["p.u."]
                max.append(self.cur_pu / unit_pu) #max according to pu
                for j in xrange(3):  #max according to resources
                    c = unit_dat[type]["cost"][j]
                    if c > 0:
                        max.append(self.cur_res[j] / c)
                av = min(max)
                ####
                if decreased:  #if nr decreased all others are INcreased
                    av += self.troops[i]
                self.avail[i] = av

    def submit(self):
        self.msgline.set_msg("")
        self.game.player.cur_res = self.cur_res
        self.region.new_troops = self.troops
        self.quit()

    def abort(self):
        self.msgline.set_msg("")
        self.quit()

    def update_screen(self):
        change = 0
        for name in self.button:
            if self.button[name].changed:
                change = 1
                self.button[name].changed = 0
                self.button[name].draw(self.bg)
        if change:
            self.conti.update()
            self.screen.blit(self.bg, (0,0))
            pygame.display.flip()
